@@ -3,30 +3,24 @@ import os.path as osp
 
 from pyboreas.data.calib import Calib
 from pyboreas.data.sensors import Camera, Lidar, Radar
-from pyboreas.utils.utils import get_closest_index
+from pyboreas.utils.utils import get_closest_frame
 
 
 class Sequence:
     """
     Class for working with an individual Boreas dataset sequence
     """
-    def __init__(self, boreas_root, seqSpec, verbose=False):
+    def __init__(self, boreas_root, seqSpec):
         """init
         Args:
             boreas_root (str): path to root folder ex: /path/to/data/boreas/
             seqSpec (list): defines sequence ID, start_time, and end_time
-            verbose (bool): whether or not to print info during initialization
         """
         self.ID = seqSpec[0]
-        self.verbose = verbose
-        if verbose:
-            print('SEQ: {}'.format(self.ID))
         if len(seqSpec) > 2:
             assert seqSpec[2] > seqSpec[1], 'Sequence timestamps must go forward in time'
             self.start_ts = str(seqSpec[1])
             self.end_ts = str(seqSpec[2])
-            if verbose:
-                print('START: {} END: {}'.format(self.start_ts, self.end_ts))
         else:
             self.start_ts = '0'  # dummy start and end if not specified
             self.end_ts = '9' * 21
@@ -43,13 +37,18 @@ class Sequence:
         # Creates list of frame objects for cam, lidar, radar, and inits poses
         self.get_all_frames()
 
-        self._check_download()  # if verbose, prints warning when sensor data missing
+        self.load_label_files()
 
-        if verbose:
-            print('camera frames: {}'.format(len(self.camera_frames)))
-            print('lidar frames: {}'.format(len(self.lidar_frames)))
-            print('radar frames: {}'.format(len(self.radar_frames)))
-            print('-------------------------------')
+        self._check_download()  # prints warning when sensor data missing
+
+    def print(self):
+        print('SEQ: {}'.format(self.ID))
+        if self.end_ts != '9' * 21:
+            print('START: {} END: {}'.format(self.start_ts, self.end_ts))
+        print('camera frames: {}'.format(len(self.camera_frames)))
+        print('lidar frames: {}'.format(len(self.lidar_frames)))
+        print('radar frames: {}'.format(len(self.radar_frames)))
+        print('-------------------------------')
 
     def get_camera(self, idx):
         self.camera_frames[idx].load_data()
@@ -177,17 +176,30 @@ class Sequence:
         lstamps = [frame.timestamp for frame in self.lidar_frames]
         rstamps = [frame.timestamp for frame in self.radar_frames]
 
-        def get_closest(query_time, target_times, targets):
-            closest = get_closest_index(query_time, target_times)
-            assert(abs(query_time - target_times[closest]) < 1.0), 'query: {}'.format(query_time)
-            return targets[closest]
-
         if ref == 'camera':
-            self.lidar_frames = [get_closest(cstamp, lstamps, self.lidar_frames) for cstamp in cstamps]
-            self.radar_frames = [get_closest(cstamp, rstamps, self.radar_frames) for cstamp in cstamps]
+            self.lidar_frames = [get_closest_frame(cstamp, lstamps, self.lidar_frames) for cstamp in cstamps]
+            self.radar_frames = [get_closest_frame(cstamp, rstamps, self.radar_frames) for cstamp in cstamps]
         elif ref == 'lidar':
-            self.camera_frames = [get_closest(lstamp, cstamps, self.camera_frames) for lstamp in lstamps]
-            self.radar_frames = [get_closest(lstamp, rstamps, self.radar_frames) for lstamp in lstamps]
+            self.camera_frames = [get_closest_frame(lstamp, cstamps, self.camera_frames) for lstamp in lstamps]
+            self.radar_frames = [get_closest_frame(lstamp, rstamps, self.radar_frames) for lstamp in lstamps]
         elif ref == 'radar':
-            self.camera_frames = [get_closest(rstamp, cstamps, self.camera_frames) for rstamp in rstamps]
-            self.lidar_frames = [get_closest(rstamp, lstamps, self.lidar_frames) for rstamp in rstamps]
\ No newline at end of file
+            self.camera_frames = [get_closest_frame(rstamp, cstamps, self.camera_frames) for rstamp in rstamps]
+            self.lidar_frames = [get_closest_frame(rstamp, lstamps, self.lidar_frames) for rstamp in rstamps]
+
+    def filter_frames_gt(self):
+        # Only keep lidar frames that were labelled, NO interpolated frames
+        keep = []
+        for frame in self.lidar_frames:
+            if frame.has_bbs():
+                keep.append(frame)
+        self.lidar_frames = keep
+
+    def load_label_files(self):
+        self.labelFiles = []
+        self.labelTimes = []
+        self.labelPoses = []
+        for frame in self.lidar_frames:
+            if frame.has_bbs():
+                self.labelFiles.append(osp.join(self.seq_root, 'labels', frame.frame + '.txt'))
+                self.labelTimes.append(frame.timestamp)
+                self.labelPoses.append(frame.pose)
\ No newline at end of file
