@@ -1,3 +1,4 @@
+from bisect import bisect_left
 import os.path as osp
 from pathlib import Path
 import numpy as np
@@ -6,6 +7,7 @@ import cv2
 
 def load_lidar(path):
     """Loads a pointcloud (np.ndarray) (N, 6) from path [x, y, z, intensity, laser_number, time]"""
+    # dtype MUST be float32 to load this properly!
     points = np.fromfile(path, dtype=np.float32).reshape((-1, 6)).astype(np.float64)
     t = get_time_from_filename(path)
     points[:, 5] += t
@@ -78,7 +80,7 @@ def get_transform2(R, t):
     return T
 
 
-def get_transform3(x, y, theta, dtype=np.float32):
+def get_transform3(x, y, theta, dtype=np.float64):
     """Returns a 4x4 homogeneous 3D transform for a given 2D (x, y, theta).
     Args:
         x (float): x-translation
@@ -219,7 +221,7 @@ def se3ToSE3(xi):
     Returns:
         np.ndarray: 4x4 transformation matrix
     """
-    T = np.identity(4, dtype=np.float32)
+    T = np.identity(4, dtype=np.float64)
     rho = xi[0:3].reshape(3, 1)
     phibar = xi[3:6].reshape(3, 1)
     phi = np.linalg.norm(phibar)
@@ -304,64 +306,11 @@ def get_time_from_filename(file):
     """Retrieves an epoch time from a file name in seconds"""
     tstr = str(Path(file).stem)
     gpstime = float(tstr)
-    timeconvert = 1e-9
-    if len(tstr) < 19:
+    timeconvert = 1e-6
+    if len(tstr) != 16 and len(tstr) > 10:
         timeconvert = 10**(-1 * (len(tstr) - 10))
     return gpstime * timeconvert
 
-
-EARTH_SEMIMAJOR = 6378137.0
-EARTH_SEMIMINOR = 6356752.0
-EARTH_ECCEN = 0.081819190842622
-a = EARTH_SEMIMAJOR
-eccSquared = EARTH_ECCEN**2
-eccPrimeSquared = (eccSquared) / (1 - eccSquared)
-k0 = 0.9996     # scale factor
-DEG_TO_RAD = np.pi / 180
-RAD_TO_DEG = 1.0 / DEG_TO_RAD
-
-
-def LLtoUTM(latitude, longitude):
-    """Converts a lat-long position into metric UTM coordinates (x-y-z)
-    Args:
-        latitude (float) in radians
-        longitude (float) in radians
-    Returns:
-        UTMEasting (float): metric position in easting
-        UTMNorthing (float): metric position in northing
-        zoneNumber (float): UTM zone number
-    """
-    while longitude < -1 * np.pi:
-        longitude += 2 * np.pi
-    while longitude >= np.pi:
-        longitude -= 2 * np.pi
-    longDeg = longitude * RAD_TO_DEG
-    latDeg = latitude * RAD_TO_DEG
-    zoneNumber = int((longDeg + 180) / 6) + 1
-    # +3 puts origin in middle of zone
-    longOrigin = (zoneNumber - 1) * 6 - 180 + 3
-    longOriginRad = longOrigin * DEG_TO_RAD
-    N = a / np.sqrt(1 - eccSquared * np.sin(latitude) * np.sin(latitude))
-    T = np.tan(latitude) * np.tan(latitude)
-    C = eccPrimeSquared * np.cos(latitude) * np.cos(latitude)
-    A = np.cos(latitude) * (longitude - longOriginRad)
-    M = a * ((1 - eccSquared / 4 - 3 * eccSquared * eccSquared / 64 - 5 * eccSquared * eccSquared * eccSquared / 256) * latitude -
-        (3 * eccSquared / 8 + 3 * eccSquared * eccSquared / 32 + 45 * eccSquared * eccSquared * eccSquared / 1024) * np.sin(2 * latitude) +
-        (15 * eccSquared * eccSquared / 256 + 45 * eccSquared * eccSquared * eccSquared / 1024) * np.sin(4 * latitude) -
-        (35 * eccSquared * eccSquared * eccSquared / 3072) * np.sin(6 * latitude))
-    UTMEasting = k0 * N * (A + (1 - T + C) * A * A * A / 6 +
-        (5 - 18 * T + T * T + 72 * C - 58 * eccPrimeSquared) * A * A * A * A * A / 120) + 500000.0
-    UTMNorthing = k0 * (M + N * np.tan(latitude) *
-        (A * A / 2 + (5 - T + 9 * C + 4 * C * C) * A * A * A * A / 24 +
-        (61 - 58 * T + T * T + 600 * C - 330 * eccPrimeSquared) * A * A * A * A * A * A / 720))
-
-    if latitude < 0:
-        # 10000000 meter offset for southern hemisphere
-        UTMNorthing += 10000000.0
-
-    return UTMEasting, UTMNorthing, zoneNumber
-
-
 def get_gt_data_for_frame(root, sensType, frame):
     """Retrieves ground truth applanix data for a given sensor frame
     Args:
@@ -381,36 +330,6 @@ def get_gt_data_for_frame(root, sensType, frame):
     return None
 
 
-def binaryDistSearch(arr, l, r, x):
-    """Performs a binary search to find the index of the element in arr
-    which is closest to x. O(log n)
-    Note: this function may also return the second-closest element to x,
-    so a follow-up check must be done on one index above and below
-    Args:
-        arr (list): Sorted list of float values
-        l (int): index of the left-most section of the array to search
-        r (int): index of the right-most section of the array to search
-        x (float): query value
-    Returns:
-        d (float): absolute distance between arr[idx] and x
-        idx (int): index of the closest element in array to x
-    """
-    if r == l + 1:
-        if abs(arr[l] - x) < abs(arr[r] - x):
-            return abs(arr[l] - x), l
-        else:
-            return abs(arr[r] - x), r
-    mid = l + (r - l) // 2
-    if arr[mid] == x:
-        return 0, mid
-    elif r - l == 0:
-        return abs(arr[l] - x), l
-    elif arr[mid] > x:
-        return binaryDistSearch(arr, l, mid - 1, x)
-    else:
-        return binaryDistSearch(arr, mid + 1, r, x)
-
-
 def get_closest_index(query, targets):
     """Retrieves the index of the element in targets that is closest to query O(log n)
     Args:
@@ -419,7 +338,11 @@ def get_closest_index(query, targets):
     Returns:
         idx (int): index of the closest element in the array to x
     """
-    d, idx = binaryDistSearch(targets, 0, len(targets)-1, query)
+    idx = bisect_left(targets, query)
+    if idx >= len(targets):
+        idx = len(targets) - 1
+    d = abs(targets[idx] - query)
+
     # check if index above or below is closer to query
     if targets[idx] < query and idx < len(targets) - 1:
         if abs(targets[idx + 1] - query) < d:
@@ -430,6 +353,32 @@ def get_closest_index(query, targets):
     return idx
 
 
+def get_closest_frame(query_time, frame_times, frames):
+    """Retrives the closest frame to query_time
+    Args:
+        query_time (float)
+        frame_times (list): list of timestamps which corresponds to the frames list
+        frames: (list): list of frames
+    Returns:
+        closest_frame (SensorType)
+    """
+    closest = get_closest_index(query_time, frame_times)
+    assert(abs(query_time - frame_times[closest]) < 3.0), 'query: {}'.format(query_time)
+    return frames[closest]
+
+
 def is_sorted(x):
     """Returns True is x is a sorted list, otherwise False"""
     return (np.diff(x) >= 0).all()
+
+
+def get_T_bev_metric(resolution, width):
+    alpha = 1 / resolution
+    if (width % 2) == 0:
+        min_range = (width / 2 - 0.5)
+    else:
+        min_range = width // 2
+    return np.array([[0, alpha, 0, min_range],
+                   [-alpha, 0, 0, min_range],
+                   [0, 0, 1, 0],
+                   [0, 0, 0, 1]])
